#ifndef PARTICLE_SYSTEM_H
#define PARTICLE_SYSTEM_H

#include "Particle.h"
#include "ParticleEmiter.h"
#include "RenderingHelper.h"
#include "Texture.h"
#include "Random.h"

#include <set>

class ParticleSystem
{
	std::set<ParticleEmiterRef> emiters;
	typedef std::set<ParticleEmiterRef>::iterator EmitersIterator;

	Particle* firstParticle;
	Particle* lastParticle;

	TextureRef texture;
	BlendMode blendMode;

	int nParticles;
	
	void AddParticle(Particle* p)
	{
		nParticles++;

		if (firstParticle)
		{
			p->nextParticle = 0;
			p->prevParticle = lastParticle;

			lastParticle->nextParticle = p;
			lastParticle = p;
		}
		else
		{
			p->nextParticle = p->prevParticle = 0;

			firstParticle = lastParticle = p;
		}
	}

	void RemoveParticle(Particle* p)
	{
		nParticles--;

		if (p->prevParticle)
			p->prevParticle->nextParticle = p->nextParticle;
		else
			firstParticle = p->nextParticle;

		if (p->nextParticle)
			p->nextParticle->prevParticle = p->prevParticle;
		else
			lastParticle = p->prevParticle;			
	}

public:
	ParticleSystem(const TextureRef& texture = TextureRef(), BlendMode blendMode = BlendModes::Normal) 
		: firstParticle(0), lastParticle(0), nParticles(0), texture(texture), blendMode(blendMode)
	{
	}

	void RemoveAllParticles()
	{
		Particle* p = firstParticle;

		while (p)
		{
			Particle* toDelete = p;

			p = p->nextParticle;

			delete toDelete;
		}

		firstParticle = 0;
		lastParticle = 0;
		nParticles = 0;
	}

	~ParticleSystem()
	{
		Particle* p = firstParticle;

		while (p)
		{
			Particle* toDelete = p;

			p = p->nextParticle;

			delete toDelete;
		}
	}

	Particle* CreateParticle()
	{
		Particle* p = new Particle();

		p->uvRegion = &DefaultRegion;

		if (p) AddParticle(p);

		return p;
	}

	Random random;

	void DestroyParticle(Particle* p)
	{
		RemoveParticle(p);

		delete p;
	}

	void Update(float dt);
	void Render();

	void AddEmiter(const ParticleEmiterRef& emiter)
	{
		emiters.insert(emiter);
	}

	void RemoveEmiter(const ParticleEmiterRef& emiter)
	{
		EmitersIterator it = emiters.find(emiter);
		if (it != emiters.end()) emiters.erase(it);
	}

	void RemoveAllEmiters()
	{
		emiters.clear();
	}

	void SetTexture(const TextureRef& t = TextureRef())
	{
		texture = t;
	}

	void SetBlendMode(BlendMode b)
	{
		blendMode = b;
	}

	int GetParticlesCount()
	{
		return nParticles;
	}

	void Reset()
	{
		RemoveAllEmiters();
		RemoveAllParticles();
	}
};

typedef boost::shared_ptr<ParticleSystem> ParticleSystemRef;

#endif